

SPECIFY_PYTHON_TO_CREATE_VIRTUALENV_WITH = """\
Create hitch virtualenv using specific python version
(e.g. /usr/bin/python3). Defaults to using python3 on the system path."""


SPECIFY_VIRTUALENV_TO_CREATE_HITCH_WITH = """\
Create hitch virtualenv using specific virtualenv
(e.g. /usr/bin/virtualenv). Defaults to using virtualenv on the system path."""


YOU_MUST_HAVE_VIRTUALENV_INSTALLED = """\
You must have virtualenv installed to use hitch.

Suggestions:

#1 Install via your system's package manager:
  - On Ubuntu/Debian : sudo apt-get install python-virtualenv
  - On Fedora        : sudo yum install python-virtualenv
  - On Arch          : sudo pacman -Sy python-virtualenv
  - On Mac OS X      : pip install --upgrade virtualenv

#2 Install via pip, e.g.:
  - sudo pip3 install --upgrade virtualenv"""


YOU_MUST_HAVE_PYTHON3_INSTALLED = """\
To use HitchKey, you must have python 3 installed.

To install:
  - On Ubuntu/Debian : sudo apt-get install python3
  - On Fedora        : sudo yum install python3
  - On Arch          : sudo pacman -Sy python3
  - On Mac OS X      : brew install python3
"""

YOU_MUST_HAVE_VERSION_ABOVE_PYTHON33 = """\
Hitch must have python 3.3 or higher installed to run.
Your app can run with earlier versions of python, but the
testing environment cannot.

Suggestions:

#1 You may need to run a sytem upgrade or upgrade your OS.
"""


HITCH_DIRECTORY_MOVED = """\
The hitch directory '{0}' was moved.
"Run 'hitch clean' then run 'hitch init' in this directory:
==> {1}
"""


HITCH_NOT_INITIALIZED = """\
Hitch has not been initialized in this directory,
or any of the directories beneath it:\n"""


SOMETHING_CORRUPTED = """\
WARNING: Hitch directory was corrupted. Run 'hitch clean' and hitch init again.
Please, if you think this is a bug, raise an issue at:\n

http://github.com/hitchtest/hitchkey/issues/
"""


SPACES_NOT_ALLOWED = """\
Spaces not allowed anywhere in the genpath:

{0}
"""



"""High level command line interface to hitch."""
from os.path import join, ismount, exists, abspath, dirname, realpath
from os import path, makedirs
from hitchkey import utils
from shutil import rmtree
import random
import string
import sys
import os


def new_hitch_dir():
    """
    Create a new named hitchdir.
    """
    return path.join(
        path.expanduser("~/.hitch"),
        ''.join([
            random.choice(
                string.digits + string.ascii_lowercase
            ) for _ in range(6)
        ])
    )


def run():
    checkdirectory = os.getcwd()
    directories_checked = []
    keypy_filename = None
    arguments = sys.argv[1:]

    python3, virtualenv = utils.check_python_and_virtualenv(
        None,
        None,
    )

    if len(arguments) > 0:
        if arguments[0] == "--quickstart":
            try:
                utils.ensure_share_directory_exists()
                if not exists(utils.quickstart_path()):
                    utils.check_call([
                        virtualenv,
                        utils.quickstart_path(),
                        "--no-site-packages",
                        "-p", python3
                    ])
                pip = join(utils.quickstart_path(), "bin", "pip")
                utils.check_call([
                    pip, "install", "pip", "--upgrade",
                ])
                utils.check_call([
                    pip, "install", "hitchqs", "--upgrade",
                ])
                quickstart = abspath(join(utils.quickstart_path(), "bin", "quickstart"))
                os.execve(quickstart, [quickstart] + arguments[1:], utils.execution_env())
                return
            except utils.CalledProcessError:
                rmtree(utils.quickstart_path(), ignore_errors=True)
                sys.exit(1)

    while not ismount(checkdirectory):
        if exists("{0}{1}key.py".format(checkdirectory, os.sep)):
            keypy_filename = "{0}{1}key.py".format(checkdirectory, os.sep)
            break
        elif exists(join(checkdirectory, "hitch", "key.py")):
            keypy_filename = join(checkdirectory, "hitch", "key.py")
            break
        else:
            directories_checked.append(join(checkdirectory, "hitch"))
            directories_checked.append(checkdirectory)
            checkdirectory = abspath(join(checkdirectory, os.pardir))

    if not keypy_filename:
        sys.stderr.write("key.py not found in the following directories:\n\n")
        sys.stderr.write('\n'.join(directories_checked))
        sys.stderr.write("\n\nCreate a key.py file in a convenient project directory to begin.\n")
        sys.exit(1)

    keypy_directory = dirname(keypy_filename)

    gensymlink = abspath(join(keypy_directory, "gen"))

    if len(arguments) > 0:
        if arguments[0] == "--clean":
            if len(arguments) == 1:
                if exists(gensymlink):
                    rmtree(realpath(gensymlink))
                    os.remove(gensymlink)
                    rmtree(abspath(join(keypy_directory, "__pycache__")), ignore_errors=True)
                    sys.exit(0)
                else:
                    print("No genfiles or pycache to clean for this project.")
                    sys.exit(1)

    if exists(gensymlink):
        genpath = realpath(gensymlink)
    else:
        genpath = new_hitch_dir()

    if not exists(genpath):
        makedirs(genpath)
        try:
            utils.check_call([
                virtualenv,
                join(genpath, "hvenv"),
                "--no-site-packages",
                "-p", python3
            ])
            utils.check_call([
                join(genpath, "hvenv", "bin", "pip"), "install", "hitchrun"
            ])
            with open(join(genpath, "hvenv", "linkfile"), 'w') as handle:
                handle.write(keypy_directory)
            os.symlink(genpath, join(keypy_directory, "gen"))
        except utils.CalledProcessError:
            rmtree(genpath, ignore_errors=True)
            sys.exit(1)
    hitchrun = abspath(join(genpath, "hvenv", "bin", "hitchrun"))
    os.execve(hitchrun, [hitchrun] + arguments, utils.execution_env())


if __name__ == '__main__':
    run()



from sys import stderr, exit
from subprocess import call, PIPE, Popen, STDOUT
from hitchkey import languagestrings
from functools import reduce
from os import path, getcwd, environ, makedirs
import shutil


class CalledProcessError(Exception):
    """Re-implemented CalledProcessError, since it is not available < python 2.7."""
    pass


def quickstart_path():
    """
    Get ~/.hitch/share/quickstart/
    """
    return path.join(path.expanduser("~"), ".hitch", "share", "quickstart")


def ensure_share_directory_exists():
    """
    Ensure that:

    ~/.hitch/ exists
    ~/.hitch/share/ exists
    """
    hitch_path = path.join(path.expanduser("~"), ".hitch")

    if not path.exists(hitch_path):
        makedirs(hitch_path)

    share_path = path.join(hitch_path, "share")

    if not path.exists(share_path):
        makedirs(share_path)


def ensure_quickstart_gone():
    """
    Ensure that ~/.hitch/share/quickstart/ virtualenv is deleted.
    """
    if path.exists(quickstart_path()):
        shutil.rmtree(quickstart_path())


def execution_env():
    """
    Return environment variables which will make subprocesses function properly.

    Avoids hashbang/wrong python env bug: https://github.com/pypa/virtualenv/issues/845
    """
    env = dict(environ)
    if "__PYVENV_LAUNCHER__" in env:
        env.pop("__PYVENV_LAUNCHER__")
    return env


def check_output(command, stdout=PIPE, stderr=PIPE):
    """Re-implemented subprocess.check_output since it is not available < python 2.7."""
    return Popen(command, stdout=stdout, stderr=stderr, env=execution_env()).communicate()[0]


def check_call(command, shell=False):
    """Re-implemented subprocess.check_call since it is not available < python 2.7."""
    process = Popen(command, shell=shell, env=execution_env())
    process.communicate()
    if process.returncode != 0:
        stderr.write("Error running {}".format(' '.join(command)))
        raise CalledProcessError
    return


def stop_everything(sig, frame):
    """Exit hitch."""
    exit(1)


def check_python_and_virtualenv(python, virtualenv):
    """Check that virtualenv and python3 are available and usable."""
    if python is None:
        if call(["which", "python3"], stdout=PIPE, stderr=PIPE) != 0:
            stderr.write(languagestrings.YOU_MUST_HAVE_PYTHON3_INSTALLED)
            stderr.flush()
            exit(1)
        python3 = check_output(["which", "python3"]).decode('utf8').replace("\n", "")
    else:
        if path.exists(python):
            python3 = python
        else:
            stderr.write("{0} not found.\n\n".format(python))
            stderr.write(languagestrings.YOU_MUST_HAVE_PYTHON3_INSTALLED)
            exit(1)

    python_version = check_output([python3, "-V"], stderr=STDOUT).decode('utf8')
    replacements = ('Python ', ''), ('\n', '')
    str_version = reduce(lambda a, kv: a.replace(*kv), replacements, python_version)
    tuple_version = tuple([int(x) for x in str_version.split('.')[:2]])

    if tuple_version < (3, 3):
        stderr.write(languagestrings.YOU_MUST_HAVE_VERSION_ABOVE_PYTHON33)
        exit(1)

    if virtualenv is None:
        if call(["which", "virtualenv"], stdout=PIPE, stderr=PIPE) != 0:
            stderr.write(languagestrings.YOU_MUST_HAVE_VIRTUALENV_INSTALLED)
            stderr.flush()
            exit(1)
        virtualenv = check_output(["which", "virtualenv"]).decode('utf8').replace("\n", "")
    else:
        if path.exists(virtualenv):
            if python is None:
                python = path.join(path.dirname(virtualenv), "python")
        else:
            stderr.write("{0} not found.\n".format(virtualenv))
            exit(1)

    return python3, virtualenv


def fail_if_spaces_in_path(filepath):
    """Raise error if spaces are found in a path."""
    if " " in filepath:
        stderr.write(languagestrings.SPACES_NOT_ALLOWED.format(getcwd()))
        exit(1)
    return filepath


def read_config(filename):
    """
    Read a standard ini file without section headers.
    """
    properties = {}
    if path.exists(filename):
        with open(filename, 'r') as handle:
            for line in "".join(handle.readlines()).split("\n"):
                if line != "" and not line.startswith(";"):
                    if line.count("=") != 1:
                        stderr.write("Error reading {0}\n".format(filename))
                        stderr.write("Line '{0}' does not follow key=val\n".format(line))
                        exit(1)
                    key, value = line.split("=")
                    properties[key] = value
    return properties



# -*- coding: utf-8 -*
from setuptools.command.install import install
from setuptools import find_packages
from setuptools import setup
from sys import version_info, stderr, exit
import codecs
import sys
import os


if sys.platform == "win32" or sys.platform == "cygwin":
    stderr.write("Hitch will not work on Windows. Sorry.\n")
    exit(1)


if version_info[0] == 2:
    if version_info[1] < 6:
        stderr.write("The hitch bootstrapper will not run on versions of python below v2.6.\n")
        exit(1)


if version_info[0] == 3:
    if version_info[1] < 3:
        stderr.write("The hitch bootstrapper will not run on python 3.0.x, 3.1.x or 3.2.x.\n")
        exit(1)


def read(*parts):
    # intentionally *not* adding an encoding option to open
    # see here: https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    return codecs.open(os.path.join(os.path.abspath(os.path.dirname(__file__)), *parts), 'r').read()


setup(name="hitchkey",
    version=read('VERSION').replace('\n', ''),
    description="HitchKey bootstrapper - set up a development environment.",
    long_description=read('README.md'),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Topic :: Software Development :: Testing',
        'Topic :: Software Development :: Libraries',
        'Operating System :: Unix',
        'Environment :: Console',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='hitch development environment command line',
    author='Colm O\'Connor',
    author_email='colm.oconnor.github@gmail.com',
    url='https://hitchtest.readthedocs.org/',
    license='AGPL',
    install_requires=[],
    packages=find_packages(exclude=["docs", ]),
    package_data={},
    entry_points=dict(console_scripts=['hk=hitchkey:commandline.run',]),
    zip_safe=False,
    include_package_data=True,
)
